
	//************************************************************
	//* Spektrum Satellite format (8-N-1/115Kbps) MSB sent first
	//* DX7/DX6i: One data-frame at 115200 baud every 22ms.
	//*           Every frame is 16 byte long and consists of a 'frame loss' counter value
	//*           plus 7 channels.
	//*
	//*    byte1:  frame loss counter
	//*    byte2:  [0 0 0 R 0 0 N1 N0]
	//*    byte3:  channel data (MSB)
	//*    byte4:  channel data (LSB)
	//*    ...
	//*    byte15: channel data (MSB)
	//*    byte16: channel data (LSB)
	//* 
	//* Channel data (16 bit):
	//*
	//*   MSB:     [F 0 C3 C2 C1 C0 D9 D8]
	//*   LSB:     [D7 D6 D5 D4 D3 D2 D1 D0]
	//* 
	//* R: 0 for 10 bit resolution 1 for 11 bit resolution channel data
	//* N1 to N0 is the number of frames required to receive all channel data. 
	//* F: 1 = indicates beginning of 2nd frame for CH8-9 (DS9 only). Otherwise zero.
	//* C3 to C0 is the channel ID. 0 to 9 (4 bit, as assigned in the transmitter).
	//* D9 to D0 is the channel data (10 bit).
	//*
	//* Channel IDs:
	//*
	//*   0: Throttle, 1: Aileron, 2: Elevator, 3: Rudder, 4: Gear, 5: Aux1, 6: Aux2.
	//*
	//* The data values can range from 0 to 1023/2047 to define a servo pulse width 
	//* from approximately 0.75ms to 2.25ms (1.50ms difference). 1.465us per digit.
	//* A value of 171/342 is 1.0 ms
	//* A value of 512/1024 is 1.5 ms
	//* A value of 853/1706 is 2.0 ms
	//* 0 = 750us, 1023/2047 = 2250us
	//*
	//************************************************************
	

	// This code only supports DSM2 1024/22ms (10 bit and all channels in a single frame, 16 bytes)
	// CH0 = Throttle, CH1 = Aileron, CH2 = Elevator, CH3 = Rudder, CH4 = Gear, etc....

IsrGetSatData:

	in SregSaver, sreg

	push zh
	push zl
	push xh
	push xl

	lds treg, SatIndex

	lds xl, tcnt1l			;X = TCNT1 - SatRxStart, SatRxStart = TCNT1
	lds xh, tcnt1h
	lds zl, SatRxStartL
	lds zh, SatRxStartH
	sts SatRxStartL, xl
	sts SatRxStartH, xh
	sub xl, zl
	sbc xh, zh
	brpl rx1

	ldz 0				;X = ABS(X)
	sub zl, xl
	sbc zh, xh
	movw x, z

rx1:	ldz 6250			;have we waited longer than 2.5ms?
	cp  xl, zl
	cpc xh, zh
	brlo rx2

	clr treg			;yes, reset index and exit
	rjmp rx3

rx2:	lds zl, SatByteArrayAddressL	;no
	lds zh, SatByteArrayAddressH

	ldx SatByte16			;end of array reached?
	cp  zl, xl
	cpc zh, xh
	brlo rx4

	lds tt, udr0			;yes, save the 16th byte to the satelite array
	sts SatByte16, tt

	ser tt				;set flag to indicate that that we have successfully received 16 bytes
	sts SatFrameValid, tt

rx3:	ldz SatByte1			;reset to start of array
	rjmp rx5

rx4:	;--- Less than 16 bytes received ---	

	lds tt, udr0			;get byte from USART buffer
	st z+, tt			;save byte to the satelite array and advance the array address pointer
	inc treg			;update index

rx5:	sts SatByteArrayAddressL, zl	;save array address pointer
	sts SatbyteArrayAddressH, zh

	sts SatIndex, treg		;save index

	clr tt				;reset timeout counter
	sts SatTimeoutCounter, tt

	pop xl
	pop xh
	pop zl
	pop zh

	out sreg, SregSaver		
	reti


	;---



GetRxChannels:

	;--- Copy channel data from the Satelite array to the Sat Channel array ---

	lds t, SatFrameValid		;check if satelite data can be used
	tst t
	brne sat4

	rjmp rx31			;skip this section if the frame is invalid

sat4:	ldx SatByte3			;skip first 2 bytes
	ldi r17, 2			;using register R17 as Byte Index

	ld t, x				;check for protocol error (e.g. DSMX)
	andi t, 0x80
	breq sat3

	rvsetflagtrue SatProtocolError
	rjmp rx31

sat3:	ldz SatChannel1L		;set SatChannel Array to 1st location

	cli
	ld yh, x+			;get MSB from the satelite array
	ld yl, x+			;get LSB from the satelite array
	lds t, SatIndex
	sei

	cp t, r17			;avoid using a mix of new and old data
	breq sat1

	mov t, yh			;the channel ID is used to store the channel value at the correct index in the Sat Channel array
	lsr t
	andi t, 0x0E			;mask Channel ID (multiplied by two and limited to 7 channels)
	inc t				;add one so we end up at SatChannelXH
	
	add zl, t			;adjust the Sat Channel array pointer to match the satelite channel ID
	brcc sat2
	inc zh

sat2:	andi yh, 0x03			;mask for D9 & D8
	st z, yh			;store and decrement pointer to reach SatChannelXL
	sbiw z, 1
	st z, yl			;store D0 to D7

sat1:	inc r17				;continue the loop until all 7 channel values have been copied
	inc r17
	cpi r17, 16
	brlt sat3


rx31:	;--- Roll ---

	ldi t, 1			;aileron has satelite channel ID 1
	rcall GetSatChannelValue
	rcall AdjustSatValue
	clr yh				;store RX value
	b16store RxRoll

	
	;--- Pitch

	ldi t, 2			;elevator has satelite channel ID 2
	rcall GetSatChannelValue
	rcall AdjustSatValue
	clr yh				;store RX value
	b16store RxPitch


	;--- Throttle ---

	ldi t, 0			;throttle has satelite channel ID 0
	rcall GetSatChannelValue

	rvsetflagfalse flagThrottleZero

	ldz 205				;X = X - 205
	sub xl, zl
	sbc xh, zh
	rcall AdjustSatValue2

	ldz 0				;X < 0 ?
	cp  xl, zl
	cpc xh, zh
	brge rx30

	ldx 0				;yes, set to zero
	rvsetflagtrue flagThrottleZero

rx30:	clr yh				;store RX value
	b16store RxThrottle


	;--- Yaw ---

	ldi t, 3			;rudder has satelite channel ID 3
	rcall GetSatChannelValue
	rcall AdjustSatValue
	clr yh				;store in RX variable
	b16store RxYaw

	
	;--- AUX ---

	ldi t, 4			;aux has satelite channel ID 4
	rcall GetSatChannelValue
	rcall AdjustSatValue

	rvsetflagfalse flagAlarmOn
	rvsetflagfalse flagSlOn
	ldz 625
	cp  xl, zl
	cpc xh, zh
	brge rx34			;is the AUX switch in ON position?

	ldz -625
	cp  xl, zl
	cpc xh, zh
	brlt rx35			;no, is it in OFF position?

rx33:	rvsetflagtrue flagAlarmOn	;no, it is in middle position
	rjmp rx36

rx34:	rvsetflagtrue flagSlOn
	
rx35:	ser t				;re-initialize the 'delay' counter used for the 'Lost Model' alarm
	sts AlarmDelay, t

rx36:	clr yh				;store RX value
	b16store RxAux


	;--- AUX2 ---

	ldi t, 5			;aux2 has satelite channel ID 5
	rcall GetSatChannelValue
	rcall AdjustSatValue

	clr yh				;store RX value
	b16store RxAux2


	;--- AUX3 ---

	ldi t, 6			;aux3 has satelite channel ID 6
	rcall GetSatChannelValue
	rcall AdjustSatValue
	
	clr yh				;store RX value
	b16store RxAux3


	;--- Use a [-100, 100] range for Aux2 and Aux3 ---

	b16ldi Temp, 0.1
	b16mul RxAux2, RxAux2, Temp
	b16mul RxAux3, RxAux3, Temp


	;--- Check RX ---

	rvbrflagfalse SatFrameValid, rx24
	rjmp rx22

rx23:	ret

rx22:	rvinc SatTimeoutCounter		;Satelite data timeout?
	rvcp SatTimeoutCounter, RxTimeoutLimit
	brlo rx23

	rvdec SatTimeoutCounter

rx24:	rvsetflagfalse SatFrameValid	;yes, set flag to false and values to zero
	b16clr RxRoll
	b16clr RxPitch
	b16clr RxThrottle
	b16clr RxYaw
	b16clr RxAux
	b16clr RxAux2
	b16clr RxAux3
	rvsetflagtrue flagThrottleZero
	rvsetflagfalse flagSlOn
	rvsetflagfalse flagAlarmOn
	ret



	;----

GetSatChannelValue:

	mov r0, t
	ldzarray SatChannel1L, 2, r0
	ld xl, z+
	ld xh, z
	andi xh, 0x03			;limit value to 1023
	ret



	;--- Adjust satelite value ---

AdjustSatValue:				;subtract Spektrum Satelite channel value offset (512) and adjust the input range

	ldz 512				;X = X - 512
	sub xl, zl
	sbc xh, zh

AdjustSatValue2:

	b16store Temper
	b16ldi Temp, 2.93		;convert range [-341, 341] to [-1000, 1000] (throttle = [0, 1880])
	b16mul Temper, Temper, Temp
	b16load Temper
	ret


	;----



